<?php

/**
 * @file
 * Defines Rijksvideo field type.
 */

/**
 * Implements hook_field_info().
 */
function rijksvideo_field_info() {
  return array(
    // We name our field as the associative name of the array.
    'rijksvideo' => array(
      'label' => 'Rijksvideo',
      'description' => t('A video file with extra information for people with disabilities.'),
      'default_widget' => 'rijksvideo_external',
      'default_formatter' => 'rijksvideo_player',
    ),
  );
}


/**
 * Implements hook_field_validate().
 */
function rijksvideo_field_validate($obj_type, $object, $field, $instance, $langcode, &$items, &$errors) {
  module_load_include('inc', 'rijksvideo', 'rijksvideo.xml_parser');
  $xml_info = array();
  if (!isset($object)) {
    return;
  }

  foreach ($items as $delta => &$item) {
    if (!empty($item['xml']) && !valid_url($item['xml'], TRUE)) {
      $errors[$field['field_name']][$langcode][$delta][] = array(
        'error' => 'rijksvideo_xml_invalid',
        'message' => t('The URL %url is invalid. Enter a fully-qualified URL, such as http://server.rijksoverheidsvideo.nl/xml/ABC-123456-7890.xml', array('%url' => $item['xml'])),
      );
    }
    elseif (!empty($item['custom_paths'])) {
      if (empty($item['video']['movie_mp4']) || $item['video']['movie_mp4'] == '') {
        $errors[$field['field_name']][$langcode][$delta][] = array(
          'error' => 'rijksvideo_mp4_missing',
          'message' => t('You need to add an MP4 file.'),
        );
      }
    }
    if (empty($item['date']) || $item['date'] == '') {
      $item['date'] = time();
    }
    elseif (!strtotime($item['date'])) {
      $errors[$field['field_name']][$langcode][$delta][] = array(
        'error' => 'rijksvideo_invalid_date',
        'message' => t('The given date is invalid.'),
      );
    }
  }
}

/**
 * Implements hook_field_widget_error().
 */
function rijksvideo_field_widget_error($element, $error) {
  switch ($error['error']) {
    case 'rijksvideo_xml_invalid':
    case 'rijksvideo_mp4_missing':
    case 'rijksvideo_invalid_date':
      form_error($element, $error['message']);
      break;
  }
}

/***********************************************************************
 *  Field Type API: Formatter
 **********************************************************************/

/**
 * Implements hook_field_formatter_info().
 */
function rijksvideo_field_formatter_info() {
  $formatters['rijksvideo_player'] = array(
    'label' => t('Rijksvideo player'),
    'field types' => array('rijksvideo'),
    'settings' => rijksvideo_default_player_settings(),
  );
  $formatters['rijksvideo_image'] = array(
    'label' => t('Image linked to content'),
    'field types' => array('rijksvideo'),
    'settings' => rijksvideo_default_image_settings(),
  );
  return $formatters;
}

/**
 * Return default field formatter settings for the video player.
 */
function rijksvideo_default_player_settings() {
  return array(
    'width' => '436',
    'height' => '260',
    'preload' => 'none',
    'controls' => 'bottom',
  );
}

/**
 * Return default field formatter settings for the thumbnail.
 */
function rijksvideo_default_image_settings() {
  return array(
    'width' => '200',
    'height' => '150',
  );
}

/**
 * Implements hook_field_formatter_view().
 */
function rijksvideo_field_formatter_view($object_type, $object, $field, $instance, $langcode, $items, $display) {
  $element = array();
  global $language;

  switch ($display['type']) {

    case 'rijksvideo_player':
      $videos = $downloads = array();
      $caption = $audiotrack = $poster = '';

      // Process files for the theme function.
      foreach ($items as $delta => $files) {
        foreach ($files as $name => $file) {
          switch ($name) {

            case 'movie_mp4':
            case 'movie_mp4hr':
              if ($file) {
                $videos['video/mp4'] = $file;
                $downloads[] = array(
                  'mime' => 'video/mp4',
                  'type' => t('Video'),
                  'uri' => $file,
                  'label' => ($name == 'movie_mp4') ? t('MPEG-4 video') : t('MPEG-4 video HQ'),
                  'duration' => $files['duration'],
                  'date' => $files['date'],
                  'size' => $files[$name . '_size'],
                );
              }
              break;

            case 'movie_webm':
              if ($file) {
                $videos['video/webm'] = $file;
                $downloads[] = array(
                  'mime' => 'video/webm',
                  'type' => t('Video'),
                  'uri' => $file,
                  'label' => t('WebM video'),
                  'duration' => $files['duration'],
                  'date' => $files['date'],
                  'size' => $files['movie_webm_size'],
                );
              }
              break;

            case 'caption':
              if ($file) {
                $caption = $file;
                $downloads[] = array(
                  'mime' => 'text/' . _rijksvideo_get_file_extension($file),
                  'type' => t('Caption'),
                  'uri' => $file,
                  'label' => t('Subtitle'),
                  'date' => $files['date'],
                );
              }
              break;

            case 'audio':
              if ($file) {
                $audiotrack = $file;
                $downloads[] = array(
                  'mime' => 'audio/mp3',
                  'type' => t('Audio track'),
                  'uri' => $file,
                  'label' => t('Audio description'),
                  'date' => $files['date'],
                );
              }
              break;

            case 'transcription':
              $transcription = $file;
              break;

            case 'thumbnail':
              $poster = $file;
              break;

          }
        }
      }

      $element[$delta] = array(
        '#theme' => 'rijksvideo',
        '#videos' => empty($videos) ? array() : $videos,
        '#transcription' => empty($transcription) ? '' : $transcription,
        '#caption' => empty($caption) ? '' : $caption,
        '#audiotrack' => empty($audiotrack) ? '' : $audiotrack,
        '#poster' => empty($poster) ? '' : $poster,
        '#files' => $files,
        '#downloads' => $downloads,
        '#options' => $display['settings'],
      );
      break;

    case 'rijksvideo_image':
      drupal_add_css(drupal_get_path('module', 'rijksvideo') . '/css/rijksvideo.theme.css');
      foreach ($items as $delta => $item) {
        $path_info = entity_uri($object_type, $object);
        $path = $path_info['path'];

        $settings = $display['settings'];

        $image = array(
          '#theme' => 'image',
          '#path' => $item['thumbnail'],
          '#alt' => t('Poster image of video "@video"', array('@video' => $object->title)),
          '#attributes' => array(
            'width' => $settings['width'],
            'height' => $settings['height'],
          ),
        );
        $element[$delta] = array(
          '#theme' => 'link',
          '#text' => drupal_render($image),
          '#path' => $path,
          '#options' => array(
            'attributes' => array(
              'class' => array(
                'rijksvideo-preview',
              ),
            ),
            'html' => TRUE,
          ),
        );
      }

      break;
  }

  return $element;
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function rijksvideo_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];
  $element = array();

  switch ($display['type']) {
    case 'rijksvideo_player':
      $element['controls'] = array(
        '#title' => t('Display control bar'),
        '#type' => 'select',
        '#options' => array('bottom', 'top', 'over', 'none'),
        '#default_value' => $settings['controls'],
      );

      $element['preload'] = array(
        '#title' => t('Preload'),
        '#type' => 'select',
        '#options' => array(
          'none' => t('No preloading'),
          'auto' => t('Preload (autoplay)'),
        ),
        '#default_value' => $settings['preload'],
      );

      $element['width'] = array(
        '#title' => t('Width'),
        '#type' => 'textfield',
        '#default_value' => $settings['width'],
      );

      $element['height'] = array(
        '#title' => t('Height'),
        '#type' => 'textfield',
        '#default_value' => $settings['height'],
      );
      break;

    case 'rijksvideo_image':
      $element['width'] = array(
        '#title' => t('Width'),
        '#type' => 'textfield',
        '#default_value' => $settings['width'],
      );

      $element['height'] = array(
        '#title' => t('Height'),
        '#type' => 'textfield',
        '#default_value' => $settings['height'],
      );
      break;
  }
  return $element;
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function rijksvideo_field_formatter_settings_summary($field, $instance, $view_mode) {
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];
  switch ($display['type']) {
    case 'rijksvideo_player':
      return t('Width: @width px, Height: @height px, Preload: @preload, Controls: @controls', array(
        '@width' => $settings['width'],
        '@height' => $settings['height'],
        '@preload' => $settings['preload'] == 'none' ? t('No') : t('Yes'),
        '@controls' => ucfirst($settings['controls']),
      ));
      break;

    case 'rijksvideo_image':
      return t('Width: @width px, Height: @height px', array(
        '@width' => $settings['width'],
        '@height' => $settings['height'],
      ));
      break;
  }
}

/***********************************************************************
 *  Field Type API: Widget
 **********************************************************************/

/**
 * Implements hook_field_widget_info().
 */
function rijksvideo_field_widget_info() {
  return array(
    'rijksvideo_external' => array(
      'label' => t('Externally hosted Rijksvideo'),
      'field types' => array('rijksvideo'),
    ),
  );
}

/**
 * Implements hook_field_widget_form().
 */
function rijksvideo_field_widget_form(&$form, &$form_state, $field, $instance, $langcode, $items, $delta, $element) {
  // $field_name is the name of the field instance. This is the
  // name that the administrator gives the field instance when
  // attaching it to an entity.
  $field_name = $instance['field_name'];

  // Is this field instance required?
  // We can use this $required value to make sub-elements of the
  // widget form required.
  $required = $element['#required'];

  // $item is where the current saved values are stored.
  $item = & $items[$delta];

  // $element is already populated with #title, #description, #delta,
  // #required, #field_name, etc.
  $element += array(
    '#type' => 'fieldset',
  );

  $custom_paths_checkbox = array(
    _rijksvideo_make_jquery_selector($field_name, $langcode, $delta, 'custom_paths') => array(
      'checked' => FALSE,
    ),
  );

  $element['xml'] = array(
    '#type' => 'textfield',
    '#title' => t('Rijksvideo XML file'),
    '#description' => t('Example: http://server.rijksoverheidsvideo.nl/xml/ABC-123456-7890.xml'),
    '#default_value' => isset($item['xml']) ? $item['xml'] : '',
    '#states' => array(
      'visible' => $custom_paths_checkbox,
    ),
  );

  $element['custom_paths'] = array(
    '#title' => t('Enter the paths of the video files manually'),
    '#type' => 'checkbox',
    '#default_value' => isset($item['custom_paths']) ? $item['custom_paths'] : FALSE,
  );

  $element['video'] = array(
    '#title' => t('Video files'),
    '#type' => 'fieldset',
    '#states' => array(
      'invisible' => $custom_paths_checkbox,
    ),
    '#description' => t('To offer video in an accessible way, it is strongly advised to upload at least a WebM, and a MP4 file.'),
    'movie_webm' => array(
      '#type' => 'textfield',
      '#title' => t('WebM'),
      '#default_value' => isset($item['movie_webm']) ? $item['movie_webm'] : '',
      '#description' => t('Strongly advised. Supported by Opera, Chrome and Firefox.'),
    ),
    'movie_mp4' => array(
      '#type' => 'textfield',
      '#title' => t('MPEG-4 Video (mp4)'),
      '#default_value' => isset($item['movie_mp4']) ? $item['movie_mp4'] : '',
      '#description' => t('Strongly advised. Supported by most browsers (except Firefox/Opera), and mobile devices.'),
    ),
    'movie_mp4hr' => array(
      '#type' => 'textfield',
      '#title' => t('High Quality MPEG-4 Video (mp4)'),
      '#default_value' => isset($item['movie_mp4hr']) ? $item['movie_mp4hr'] : '',
    ),
  );

  $element += array(
    'transcription' => array(
      '#type' => 'textarea',
      '#title' => t('Transcription'),
      '#description' => t('If left empty, the transcription of the XML will be fetched (if available).'),
      '#rows' => 2,
      '#default_value' => $items[$delta]['transcription'],
    ),
    'thumbnail' => array(
      '#type' => 'textfield',
      '#title' => t('Thumbnail'),
      '#default_value' => $items[$delta]['thumbnail'],
      '#states' => array(
        'invisible' => $custom_paths_checkbox,
      ),
    ),
    'caption' => array(
      '#type' => 'textfield',
      '#title' => t('Caption (srt)'),
      '#default_value' => $items[$delta]['caption'],
      '#states' => array(
        'invisible' => $custom_paths_checkbox,
      ),
    ),
    'audio' => array(
      '#type' => 'textfield',
      '#title' => t('Audio description (mp3)'),
      '#default_value' => $items[$delta]['audio'],
      '#states' => array(
        'invisible' => $custom_paths_checkbox,
      ),
    ),
    'date' => array(
      '#type' => 'textfield',
      '#title' => t('Creation date'),
      '#description' => t('Example: 2011-11-17T15:53:40'),
      '#default_value' => isset($items[$delta]['date']) ? date_iso8601($items[$delta]['date']) : '',
      '#states' => array(
        'invisible' => $custom_paths_checkbox,
      ),
    ),
    'duration' => array(
      '#type' => 'textfield',
      '#title' => t('Video duration (hh:mm:ss)'),
      '#default_value' => $items[$delta]['duration'],
      '#states' => array(
        'invisible' => $custom_paths_checkbox,
      ),
    ),
  );

  return $element;
}

/**
 * Returns a jQuery selector that Drupal will use
 * to test for #states conditions.
 */
function _rijksvideo_make_jquery_selector($field_name, $langcode, $delta, $item_name) {
  $lastpart = implode('][', array($langcode, $delta, $item_name));
  return ':input[name="' . $field_name . '[' . $langcode . '][' . $delta . '][' . $item_name . ']"], ' .
    ':input[name$="[' . $field_name . '][' . $langcode . '][' . $delta . '][' . $item_name . ']"]';
}

/**
 * Implements hook_field_presave().
 */
function rijksvideo_field_presave($entity_type, $entity, $field, $instance, $langcode, &$items) {
  module_load_include('inc', 'rijksvideo', 'rijksvideo.xml_parser');

  foreach ($items as $delta => &$item) {
    // Fetch the XMl data and add it to the field item.
    if (!$item['custom_paths'] && !empty($item['xml'])) {
      $xml_info = _rijksvideo_parse_xml($item['xml']);
      // ?? $xml_info['transcription'] = $item['transcription'];
      $item = array_merge($item, $xml_info);
      if (!empty($item['video'])) {
        // Move the items within the fieldset to the parent level.
        foreach ($item['video'] as $type => $video) {
          $item['movie_' . drupal_strtolower($type)] = $video['url'];
          $item['movie_' . drupal_strtolower($type) . '_size'] = $video['size'];
        }
      }
    }
    // Custom paths data entered
    elseif ($item['custom_paths'] && !empty($item['video'])) {
      // Move the items within the fieldset to the parent level.
      foreach ($item['video'] as $type => $video) {
        $item[drupal_strtolower($type)] = $video;
        //$item[drupal_strtolower($type) . '_size'] = $video['size']; // Leave size empty for now

        global $base_url;
        // Check if this is a localy hosted video
        if (strstr($base_url, $video)) {
          $item[$type] = $video;
          $absolutepath = str_replace($base_url . '/', '', $video);
          $item[$type . '_size'] = filesize($absolutepath);
        }
      }
    }

    // Make sure the date is a timestamp
    if (!empty($item['date']) && !is_numeric($item['date'])) {
      $item['date'] = strtotime($item['date']);
    }
    if (empty($item['date']) || $item['date'] == '') {
      $item['date'] = time();
    }
  }
}

/**
 * Implements hook_field_is_empty().
 */
function rijksvideo_field_is_empty($item, $field) {
  if (empty($item['custom_paths'])) {
    return empty($item['xml']);
  }
  else {
    return empty($item['video']['movie_mp4']);
  }
}

/**
 * Implements hook_field_settings_form().
 */
function rijksvideo_field_settings_form() {
  return array();
}
